
/**
 * Created by IntelliJ IDEA.
 * User: Patricia
 * Date: Mar 7, 2004
 * Time: 5:31:06 PM
 * To change this template use File | Settings | File Templates.
 */
// Copyright (c) 2000 Just Objects B.V. <just@justobjects.nl>
// Distributable under LGPL license. See terms of license at gnu.org.


/**
 * Abstract Event source from which Events are pulled.
 *
 * @version $Id: EventPullSource.java,v 1.4 2003/08/15 08:37:40 justb Exp $
 * @author Just van den Broecke - Just Objects &copy;
 **/

/** ABC for specifc EventPullSources. */
abstract public class EventPullSource implements EventSource, Runnable {
        private volatile boolean alive = false;
        private volatile boolean active = false;
        private static int threadNum = 0;

        public EventPullSource() {
        }

        abstract protected long getSleepTime();

        abstract protected Event pullEvent();

        public void start() {
                Thread t = new Thread(this, "EventGenerator" + (++threadNum));
                t.setDaemon(true);
                t.start();
        }

        public boolean isAlive() {
                return alive;
        }

        /** Stop the event generator thread. */
        public void kill() {
                alive = false;
        }

        /** Activate the event generator thread. */
        synchronized public void activate() {
                if (active) return;
                active = true;
                if (!alive) {
                        start();
                        return;
                }
                Log.debug(getClass().getName() + ": notifying...");
                notifyAll();
        }

        /** Deactivate the event generator thread. */
        public void passivate() {
                if (!active) {
                        return;
                }
                active = false;
        }

        /** Main loop: sleep, generate event and publish. */
        public void run() {
                Log.debug(getClass().getName() + ": starting...");
                alive = true;
                while (alive) {
                        try {

                                Thread.sleep(getSleepTime());

                                // If passivated wait until we get
                                // get notify()-ied. If there are no subscribers
                                // it wasts CPU to remain producing events...
                                synchronized (this) {
                                        while (!active) {
                                                Log.debug(getClass().getName() + ": waiting...");
                                                wait();
                                        }
                                }

                        }
                        catch (InterruptedException e) {
                        }

                        // Derived class should produce an event.
                        Event event = pullEvent();

                        // Let the publisher push it to subscribers.
                        Publisher.getInstance().publish(event);
                }
        }
}

/*
  * $Log: EventPullSource.java,v $
  * Revision 1.4  2003/08/15 08:37:40  justb
  * fix/add Copyright+LGPL file headers and footers
  *
  * Revision 1.3  2003/08/12 09:57:05  justb
  * replaced all print statements to Log.*() calls
  *
  * Revision 1.2  2003/05/18 16:15:08  justb
  * support for XML encoded Events
  *
  * Revision 1.1.1.1  2002/09/24 21:02:31  justb
  * import to sourceforge
  *
  * Revision 1.1.1.1  2002/09/20 22:48:17  justb
  * import to SF
  *
  * Revision 1.1.1.1  2002/09/20 14:19:03  justb
  * first import into SF
  *
  * Revision 1.3  2002/04/15 20:42:41  just
  * reformatting and renaming GuardedQueue to BlockingQueue
  *
  * Revision 1.2  2000/08/21 20:48:29  just
  * added CVS log and id tags plus copyrights
  *
  *
  */
